/*
 * ByteArray.java
 *
 * Created on 7. Juni 2007, 20:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package network;

import java.util.Iterator;
import java.util.Vector;
import softwareprojekt.Form;
import softwareprojekt.Lecture;
import softwareprojekt.LectureDate;
import softwareprojekt.User;

/** Objekt ist ein Bytearray mit Eigenschaften zum umwandeln des Inhalts.
 *
 * @author PMV
 */
public class ByteArray {
    private byte [] array = null;
    
    //Dummy-Methode zum erstellen eines ByteArrays aus einem Vektor
    private static byte [] vectorToByteArray(Vector vector){
        byte [] array = new byte [vector.size()];
        for (int i = 0; i < vector.size(); i++){
            array[i] = ((Byte) vector.get(i)).byteValue();
        }
        return array;
    }
    
    /** Erstellt aus einem Bytearray ein ByteArray-Objekt */
    public ByteArray(byte [] array){
        this.array = array.clone();
    }
    
    /** Erstellst aus einem String ein ByteArray-Objekt
     * Hat die selbe Bedeutung wie der Aufruf<br>
     * new ByteArray(string, string.length());
     */
    public ByteArray(String string){
        this.array = string.getBytes();
    }
    
    /** Erstellt aus einem String ein ByteArray-Objekt
     * Die der bergebene String krzer als 'length', so wird der
     * String vorher mit vorrangestellten Leerzeichen aufgefllt.<br>
     * Ist der String grer als 'length', werden die hinteren Zeichen
     * abgeschnitten.
     */
    public ByteArray(String string, int length){
        if (string == null){
            string = " ";
        }
        
        if (string.length() > length){
            string = string.substring(0, length);
        } else if (string.length() < length){
            string = String.format("%1$" + length + "s", string);
        }
        this.array = string.getBytes();
    }
    
    /** Erstellt aus einem Integer-Wert ein ByteArray-Objekt (4 Elemente) */
    public ByteArray(int value){
        this.array = new byte [4];
        this.array[0] = (byte) (value >> 24);
        this.array[1] = (byte) (value >> 16);
        this.array[2] = (byte) (value >> 8);
        this.array[3] = (byte) value;
    }
    
    /** Erstellt aus einem Short-Wert ein ByteArray-Objekt (2 Elemente) */
    public ByteArray(short value){
        this.array = new byte [2];
        this.array[0] = (byte) (value >> 8);
        this.array[1] = (byte) value;
    }
    
    /** Erstellt aus einem Byte-Vektor ein ByteArray-Objekt */
    public ByteArray(Vector vector){
        this.array = vectorToByteArray(vector);
    }
    
    /** Erstellt aus einem User-Objekt ein ByteArray-Objekt */
    public ByteArray(User user){
        ByteArray temp;
        Vector vector = new Vector();
        
        //10 Bytes -> enthalten den Benutzernamen
        temp = new ByteArray(user.getID(), 10);
        temp.addToVector(vector);
        
        //40 Bytes -> enthalten den Familenname
        temp = new ByteArray(user.getLastname(), 40);
        temp.addToVector(vector);
        
        //20 Bytes -> enthalten den Vornamen
        temp = new ByteArray(user.getFirstname(), 20);
        temp.addToVector(vector);
        
        //1 Byte  -> enthlt den Benutzertyp, 'A', 'D' oder 'S'
        vector.add((byte) user.getTyp());
        
        //3 Bytes -> enthalten den Studiengang
        temp = new ByteArray(user.getStudypath(), 3);
        temp.addToVector(vector);
        
        //2 Bytes -> enthalten das Semester (short)
        temp = new ByteArray((short) user.getSemester());
        temp.addToVector(vector);
        
        this.array = vectorToByteArray(vector);
    }
    
    /** Erstellt aus einem Form-Objekt ein ByteArray-Objekt
     * Ist 'withAnswer' = false werden die enthaltenden
     * Antwortanzahlen ignoriert und dem ByteArray nicht hinzugefgt.
     */
    public ByteArray(Form form, boolean withAnswer){
        ByteArray temp;
        Vector vector = new Vector();
        String [] question = form.getQuestion();
        
        //ersten 20 Zeichen enthalten die ID
        temp = new ByteArray(form.getID(), 20);
        temp.addToVector(vector);
        try {
            for (int i = 0; i < 20; i++){
                //4 Bytes -> enthalten die Anzahl der Antworten 
                //                  fr die Frage 1
                if (withAnswer){
                    for (int ii = 0; ii < 5; ii++){
                        temp = new ByteArray(form.getAnswer(i, ii));
                        temp.addToVector(vector);
                    }
                }
                //4 Bytes -> enthalten die Lnge des Fragetextes (= len)
                temp = new ByteArray(question[i].length());
                temp.addToVector(vector);
                
                //len Bytes -> enthalten den Fragetext der Frage
                temp = new ByteArray(question[i]);
                temp.addToVector(vector);
            }
            this.array = vectorToByteArray(vector);
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
            return;
            
        }
    }
    
    /** Erstellt aus einem Lecture-Objekt ein ByteArray-Objekt. */
    public ByteArray(Lecture lecture){
        ByteArray temp;
        Vector vector = new Vector();
        
	//5 Bytes -> enthalten den Kurznamen
	temp = new ByteArray(lecture.getShortname(), 5);
        temp.addToVector(vector);
        
        //40 Bytes -> enthalten den Langnamen
	temp = new ByteArray(lecture.getName(), 40);
        temp.addToVector(vector);
        
        //3 Bytes -> enthalten den Studiengang
	temp = new ByteArray(lecture.getStudypath(), 3);
        temp.addToVector(vector);
        
        //2 Bytes -> enthalten das Semester (short)
        temp = new ByteArray((short) lecture.getSemester());
        temp.addToVector(vector);
        
        //10 Bytes -> enthalten die UserID des zugehrigen Dozenten
        temp = new ByteArray(lecture.getProf().getID(), 10);
        temp.addToVector(vector);
        
        Vector lectureDate = lecture.getLectureDate();
	
        //1 Byte  -> enthlt die Anzahl der Vorlesungstermine (= x)
        vector.add((byte) lectureDate.size());
        
        for (Iterator it = lectureDate.iterator(); it.hasNext();) {
            LectureDate date = (LectureDate) it.next();
            
            //2 Bytes -> enthalten den Index des Termins (short) 
            temp = new ByteArray((short) date.getIndex());
            temp.addToVector(vector);
            
            //2 Bytes -> enthalten die Tagkennung
            temp = new ByteArray(date.getDay(), 2);
            temp.addToVector(vector);
            
            //2 Bytes -> Beginn der Vorlesung, Uhrzeit (short)
            temp = new ByteArray((short) date.getStartTime());
            temp.addToVector(vector);
            
            //2 Bytes -> Ende der Vorlesung, Uhrzeit (short)
            temp = new ByteArray((short) date.getEndTime());
            temp.addToVector(vector);
            
            //5 Bytes -> Raumkennung
            temp = new ByteArray(date.getRoom(), 5);
            temp.addToVector(vector);
        }
        
        //ab hier muss der Fragebogen ohne Antwortzhler integriert werden
        temp = new ByteArray(lecture.getForm(), false);
        temp.addToVector(vector);
        
        this.array = vectorToByteArray(vector);

    }
    
    /** Fgt die Elemente des Arrays zu einem Vector hinzu */
    public void addToVector(Vector vector){
        for (int i = 0; i < this.array.length; i++){
            vector.add(this.array[i]);
        }
    }
    
    /** Erstellt aus 4 Element (Bytes) ein Integer-Wert
     * Index gibt das erste Element an, welches zugleich das High-Byte ist.
     * Die 3 folgenden Elemente werden dann dementsprechend dahinter gesetzt.
     */
    public int toInt(int index){
        int result = this.array[index] << 24;
        result |= (this.array[index + 1] << 16) & 16777215;
        result |= (this.array[index + 2] << 8) & 65535;
        result |= this.array[index + 3] & 255;
        //System.out.println("result: " + String.format("%1$32s", Integer.toBinaryString(result)));
        return result;
    }
    
    /** Erstellt aus 2 Element (Bytes) ein Integer-Wert
     * Index gibt das erste Element an, welches zugleich das High-Byte ist.
     * Das folgende Element ist somit das Low-Byte.
     * Der zurckgegebene Wert ist "unsignet".
     */
    public int toShort(int index){
        int result = (this.array[index] << 8 ) & 65535;
        result |= (this.array[index + 1]) & 255;
        return result & 65535;
    }
    
    /** Erstellt aus 1 Element (Byte) ein Integer-Wert
     * Index gibt das Element an, welches ausgelesen werden soll.
     * Der zurckgegebene Wert ist "unsignet".
     */
    public int toByte(int index){
        return this.array[index] & 255;
    }
    /** Wandelt das Array in einen String um.
     * 1 Byte steht fr 1 Zeichen.
     */
    public String toString(){
        return toString(0, this.array.length);
    }
    
    /** Wandelt einen Teil des Arrays in einen String um.
     * 1 Byte steht fr 1 Zeichen.
     */
    public String toString(int beginIndex, int endIndex){
        String string = "";
        
        for (int i = beginIndex; i < endIndex; i++){
            string = string + Character.toString((char) this.array[i]);
        }
        
        return string;
    }
    
    /** Wandelt das Array in ein User-Objekt um */
    public User toUser(){
        //10 Bytes -> enthalten den Benutzernamen
        String id = this.toString(0, 10).trim();
        //40 Bytes -> enthalten den Familenname
        String lastname = this.toString(10, 50).trim();
        //20 Bytes -> enthalten den Vornamen
        String firstname = this.toString(50, 70).trim();
        //1 Byte  -> enthlt den Benutzertyp, 'A', 'D' oder 'S'
        char typ = (char) this.array[70];
        //3 Bytes -> enthalten den Studiengang
        String studypath = this.toString(71, 74);
        //2 Bytes -> enthalten das Semester (short)
        int semester = this.toShort(74);
        
        switch(typ){
            case 'A':
                return User.newAdmin(id, lastname, firstname);
            case 'D':
                return User.newProf(id, lastname, firstname, studypath);
            case 'S':
                return User.newStudy(
                        id, lastname, firstname, studypath, semester);
        }
        return null;
    }
    
    
    /** Wandelt das Array in ein Form-Objekt um. (ohne Antwortanzahl)
     * Das ByteArray-Objekt darf keine Antwortanzahlen enthalten.
     */
    public Form toFormSimple(){
        return this.toFormSimple(0);
    }
    
    //liet das Fragebogen-Objekt aus dem ByteArray an einer bestimmten 
    //      Position. Die Antwortanzahl darf nicht enthalten sein.
    private Form toFormSimple(int seek){
        String [] question = new String[20];
        //ersten 20 Zeichen enthalten die ID
        String id = this.toString(seek, seek + 20).trim();
        seek += 20;
        try {
            for (int i = 0; i < 20; i++){
                //4 Bytes -> enthalten die Lnge des Fragetextes (= len)
                int len = this.toInt(seek);
                seek += 4;
                
                //len Bytes -> enthalten den Fragetext der Frage
                question[i] = this.toString(seek, seek + len);
                seek += len;
            }
            Form form = new Form(id, question);
            return form;
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
            return null;
        }
    }
    /** Wandelt das Array in ein Form-Objekt um. (mit Antwortanzahl)
     * Das ByteArray-Objekt muss die Antwortanzahl enthalten.
     */
    public Form toForm(){
        String [] question = new String[20];
        int [][] answer = new int [20][5];
        int seek = 0;
        //ersten 20 Zeichen enthalten die ID
        String id = this.toString(seek, seek + 20).trim();
        seek += 20;
        try {
            for (int i = 0; i < 20; i++){
                for (int ii = 0; ii < 5; ii++){
                    //4 Bytes -> enthalten die Anzahl der Antworten fr die Frage
                    answer[i][ii] = this.toInt(seek);
                    seek += 4;
                } // ... (5 mal)
                
                //4 Bytes -> enthalten die Lnge des Fragetextes (= len)
                int len = this.toInt(seek);
                seek += 4;
                
                //len Bytes -> enthalten den Fragetext der Frage
                question[i] = this.toString(seek, seek + len);
                seek += len;
            } // ... (20 mal)
            
            return new Form(id, question, answer);
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
            return null;
        }
    }
   
    /** Wandelt das Array in ein Lecture-Objekt um. */
    public Lecture toLecture(){
        //5 Bytes -> enthalten den Kurznamen
        String shortname = this.toString(0, 5);
        //40 Bytes -> enthalten den Langnamen
        String name = this.toString(5, 45).trim();
        //3 Bytes -> enthalten den Studiengang
        String studypath = this.toString(45, 48);
        //2 Bytes -> enthalten das Semester (short)
        int semester = this.toShort(48);
        //10 Bytes -> enthalten die UserID des zugehrigen Dozenten
        String prof = this.toString(50, 60).trim();
        
        Lecture lecture = new Lecture(
                shortname,
                name, 
                semester,
                studypath,
                User.newProf(prof, "<Test>", "", studypath));

        //1 Byte  -> enthlt die Anzahl der Vorlesungstermine (= x)
        int len = this.toByte(60);
        int seek = 61;
        for (int i = 0; i < len; i++){
            //2 Bytes -> enthalten den Index des Termins (short)
            int index = this.toShort(seek);
            seek += 2;
            //2 Bytes -> enthalten die Tagkennung
            String day = this.toString(seek, seek + 2);
            seek += 2;
            //2 Bytes -> Beginn der Vorlesung, Uhrzeit (short)
            int startTime = this.toShort(seek);
            seek += 2;
            //2 Bytes -> Ende der Vorlesung, Uhrzeit (short)
            int endTime = this.toShort(seek);
            seek += 2;
            //5 Bytes -> Raumkennung
            String room = this.toString(seek, seek + 5);
            seek += 5;
            
            lecture.addLectureDate(
                    new LectureDate(index, day, startTime, endTime, room));
        }
        
        //ab hier enthlt die Nachricht einen Fragebogen ohne Antwortzhler.
        lecture.setForm(this.toFormSimple(seek));
        return lecture;
    }
    
    /** Gibt die Lnge des Arrays zurck */
    public int length() {
        return array.length;
    }
    
    /** Berechnet aus dem Array einen gltigen Schlssel */
    public int getKey() {
        return Math.abs(this.length() / 5);
    }
    
    /** Gibt das Array selber zurck */
    public byte [] getArray() {
        return array;
    }
}
