/*
 * SendData.java
 *
 * Created on 9. Juni 2007, 20:40
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package network;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;
import softwareprojekt.Form;
import softwareprojekt.Lecture;
import softwareprojekt.User;

/** Protokoll-Objekt zum senden von Daten
 *
 * @author PMV
 */
public class SendData extends Data{
    DataOutputStream writer = null;
    
    /** Erstellt ein neues Objekt von SendData */
    public SendData(DataOutputStream writer) {
        this.writer = writer;
    }
    
    /** Sendet die zuletzt erstellten Daten */ 
    public boolean send() {
        try {
            writer.writeInt(this.data.getKey());
            writer.writeInt(this.data.length());
            writer.write((byte) this.typ);
            writer.write(this.data.getArray());
            writer.flush();
            return true;
        } catch (IOException ex) {
            ex.printStackTrace();
            return false;
        }
    }
    
    /** Erstellt die Login-Nachricht */
    public void makeLogin(String userID, String password, char userTyp){
        //alle Attribute resetten
        this.reset();
        
        ByteArray temp;
        Vector vector = new Vector();
        
        //10 Bytes -> enthalten den Benutzernamen
        temp = new ByteArray(userID, 10);
        temp.addToVector(vector);
        
        //32 Bytes -> enthalten das verschlsselte Passwort
        temp = new ByteArray(password, 32);
        temp.addToVector(vector);
        
        //1 Byte -> enthlt den erwarteten Benutzertyp
        vector.add((byte) userTyp);
        
        this.data = new ByteArray(vector);
        this.typ = 'i';
    }
    
    /** Erstellt die Logout-Nachricht */
    public void makeLogout(){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(" ".getBytes());
        this.typ = 'o';
    }
    
    /** Erstellt die Ok-Nachricht */
    public void makeOk(char oldTyp){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(String.valueOf(oldTyp));
        this.typ = 'O';
    }
    
    /** Erstellt die Fehler-Nachricht */
    public void makeError(String text){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(text.getBytes());
        this.typ = 'E';
    }
    
    /** Erstellt aus einem User-Objekt die entsprechende Nachricht */
    public void makeUser(User user){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(user);
        this.typ = 'u';
    }
    
    /** Erstellt aus einem Fragebogen-Objekt die entsprechende Nachricht
     * Die resultierende Nachricht enthlt keine Antwortzahlen des
     * Fragebogens.
     */
    public void makeFormSimple(Form form){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(form, false);
        this.typ = 's';
    }
    
    /** Erstellt aus einem Fragebogen-Objekt die entsprechende Nachricht 
     * Die resultierende Nachricht enthlt die Antwortzahlen des
     * Fragebogens.*/
    public void makeForm(Form form){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(form, true);
        this.typ = 'f';
    }
    
    /** Erstellt aus einem Vorlesungs-Objekt die entsprechende Nachricht */
    public void makeLecture(Lecture lecture){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(lecture);
        this.typ = 'l';
    }
    
    /** Erstellt die "Wunsch-Nachtricht" nach einer gewnschten ID-Liste */
    public void makeIDListRequest(char typ){
        //alle Attribute resetten
        this.reset();
        
        this.data = new ByteArray(String.valueOf(typ));
        this.typ = 'L';
    }
    
    /** Erstellt aus einem String-Array aus IDs die entsprechende Nachricht */
    public void makeIDList(String [] id, int length){
        //alle Attribute resetten
        this.reset();
        
        ByteArray temp;
        Vector vector = new Vector();
        
        //4 Bytes -> enthalten die Anzahl der IDs (Integer)
        temp = new ByteArray(id.length);
        temp.addToVector(vector);
        
        //1 Byte  -> enthlt die Anzahl der Zeichen pro ID (= y)
        vector.add((byte) length);
        
        for(int i = 0; i < id.length; i++){
            //y Bytes -> enthalten die IDs ohne Trennzeichen
            temp = new ByteArray(id[i], length);
            temp.addToVector(vector);
        }
        
        this.data = new ByteArray(vector);
        this.typ = 'l';
    }
    
    /** Erstellt aus zwei Strings die Passwortndern-Nachricht */
    public void makeChangePwd(String oldPwd, String newPwd){
        //alle Attribute resetten
        this.reset();
        
        String string = String.format("%1$32s", oldPwd);
        string += String.format("%1$32s", newPwd);
        System.out.println(string);
        this.data = new ByteArray(string);
        this.typ = 'p';
    }
    
}
