/*
 * ByteArrayTest.java
 * JUnit based test
 *
 * Created on 10. Juni 2007, 23:46
 */

package network;

import junit.framework.*;
import java.util.Vector;
import softwareprojekt.Form;
import softwareprojekt.Lecture;
import softwareprojekt.LectureDate;
import softwareprojekt.User;

/**
 *
 * @author PMV
 */
public class ByteArrayTest extends TestCase {
    
    public ByteArrayTest(String testName) {
        super(testName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(ByteArrayTest.class);
        
        return suite;
    }

    /**
     * Test of addToVector method, of class network.ByteArray.
     */
    public void testAddToVector() {
        System.out.println("addToVector");
        
        Vector vector = new Vector();
        ByteArray instance = new ByteArray("addToVector");
        
        instance.addToVector(vector);
        assertEquals(instance.toString(), new ByteArray(vector).toString());
        

    }

    /**
     * Test of toInt method, of class network.ByteArray.
     */
    public void testToInt() {
        System.out.println("toInt");
        
        int value = 1043345423;
        ByteArray instance = new ByteArray(value);
        
        int result = instance.toInt(0);
        assertEquals(value, result);
    }

    /**
     * Test of toShort method, of class network.ByteArray.
     */
    public void testToShort() {
        System.out.println("toShort");
        
        int value = 65500;
        ByteArray instance = new ByteArray((short) value);
        
        int result = instance.toShort(0);
        assertEquals(value, result);
        
    }

    /**
     * Test of toString method, of class network.ByteArray.
     */
    public void testToString() {
        System.out.println("toString");
        
        String expResult = "bla blub bla";
        ByteArray instance = new ByteArray(expResult);
        
        String result = instance.toString();
        assertEquals(expResult, result);
    }

    /**
     * Test of toUser method, of class network.ByteArray.
     */
    public void testToUser() {
        System.out.println("toUser");
        
        User expResult = User.newAdmin("admin", "admin", "");
        ByteArray instance = new ByteArray(expResult);
        
        User result = instance.toUser();
        assertEquals(expResult.getID(), result.getID());
        assertEquals(expResult.getFirstname(), result.getFirstname());
        assertEquals(expResult.getLastname(), result.getLastname());
        assertEquals(expResult.getTyp(), result.getTyp());
        assertEquals(expResult.getStudypath(), result.getStudypath());
        assertEquals(expResult.getSemester(), result.getSemester());

    }

    /**
     * Test of toFormSimple method, of class network.ByteArray.
     */
    public void testToFormSimple() {
        System.out.println("toFormSimple");
        
        String [] question = new String [20];
        for (int i = 0; i < 20; i++){
            question[i] = "Dies ist Frage Nummer " + i + ".";
        }
        
        Form expResult = new Form("12345", question);
        ByteArray instance = new ByteArray(expResult, false);
        Form result = instance.toFormSimple();
        
        assertEquals(expResult.getID(), result.getID());
        for (int i = 0; i < 20 ; i++){
            assertEquals(expResult.getQuestion(i), result.getQuestion(i));
        }
    }

    /**
     * Test of toForm method, of class network.ByteArray.
     */
    public void testToForm() {
        System.out.println("toForm");
        
        String [] question = new String [20];
        for (int i = 0; i < 20; i++){
            question[i] = "Dies ist Frage Nummer " + i + ".";
        }
        
        int [][] answer = new int [20][5];
        for (int i = 0; i < 20; i++){
            for (int ii = 0; ii < 5; ii++){
                answer[i][ii] = (int) (Math.random() * 1000);
            }
        }
                
        Form expResult = new Form("12345", question, answer);
        ByteArray instance = new ByteArray(expResult, true);
        Form result = instance.toForm();
        
        assertEquals(expResult.getID(), result.getID());
        for (int i = 0; i < 20 ; i++){
            assertEquals(expResult.getQuestion(i), result.getQuestion(i));
            for (int ii = 0; ii < 5; ii++){
                assertEquals(expResult.getAnswer(i, ii), result.getAnswer(i, ii));
            }
        }
    }

    /**
     * Test of toLecture method, of class network.ByteArray.
     */
    public void testToLecture() {
        System.out.println("toLecture");
        
        Vector lectureDate = new Vector();
        lectureDate.add(new LectureDate(1, "Mi", 10000, 20000, "1/333"));
        lectureDate.add(new LectureDate(1, "Do", 20000, 30000, "1/333"));
        
        Lecture expResult = new Lecture(
                "12345",
                "Dies ist eine Testvorlesung",
                4,
                "IF2",
                User.newProf("Professor", "Lastname", "", "IF2"),
                new Form("54321"),
                lectureDate);
        
        ByteArray instance = new ByteArray(expResult);
        Lecture result = instance.toLecture();
        assertEquals(expResult.getName(), result.getName());
        assertEquals(expResult.getShortname(), result.getShortname());
        assertEquals(expResult.getStudypath(), result.getStudypath());
        assertEquals(expResult.getForm().getID(),
                     result.getForm().getID());
        assertEquals(expResult.getProf().getID(),
                     result.getProf().getID());
    }
}
